@@ -1,4 +1,4 @@
-# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,13 +10,13 @@ package Devel::Cover;
 use strict;
 use warnings;
 
-our $VERSION = "0.72";
+our $VERSION = "0.74";
 
 use DynaLoader ();
 our @ISA = "DynaLoader";
 
-use Devel::Cover::DB  0.72;
-use Devel::Cover::Inc 0.72;
+use Devel::Cover::DB  0.74;
+use Devel::Cover::Inc 0.74;
 
 use B qw( class ppname main_cv main_start main_root walksymtable OPf_KIDS );
 use B::Debug;
@@ -294,6 +294,11 @@ sub import
         warn __PACKAGE__ . ": Unknown option $_ ignored\n";
     }
 
+    my $ci     = $^O eq "MSWin32";
+    @Select_re = map qr/$_/,                           @Select;
+    @Ignore_re = map qr/$_/,                           @Ignore;
+    @Inc_re    = map $ci ? qr/^\Q$_\//i : qr/^\Q$_\//, @Inc;
+
     bootstrap Devel::Cover $VERSION;
 
     if (defined $Dir)
@@ -311,9 +316,9 @@ sub import
 
     unless (-d $DB)
     {
-        mkdir $DB, 0700 or croak "Can't mkdir $DB: $!\n";
+        mkdir $DB, 0700 or die "Can't mkdir $DB: $!";
     }
-    $DB = $1 if Cwd::abs_path($DB) =~ /(.*)/;
+    $DB = $1 if abs_path($DB) =~ /(.*)/;
     Devel::Cover::DB->delete($DB) unless $Merge;
 
     if ($blib)
@@ -323,10 +328,6 @@ sub import
         push @Ignore, "^t/", '\\.t$', '^test\\.pl$';
     }
 
-    my $ci = $^O eq "MSWin32";
-    @Select_re = map qr/$_/,                           @Select;
-    @Ignore_re = map qr/$_/,                           @Ignore;
-    @Inc_re    = map $ci ? qr/^\Q$_\//i : qr/^\Q$_\//, @Inc;
     %Files     = ();  # start gathering file information from scratch
 
     for my $c (Devel::Cover::DB->new->criteria)
@@ -433,22 +434,30 @@ sub normalised_file
     }
     if ($] >= 5.008)
     {
-        if ($^O eq "MSWin32" || $^O eq "cygwin")
+        my $inc;
+        $inc ||= $file =~ $_ for @Inc_re;
+        # warn "inc for [$file] is [$inc] @Inc_re";
+        if ($inc && ($^O eq "MSWin32" || $^O eq "cygwin"))
         {
-            # TODO - Windows seems busted here
+            # Windows' Cwd::_win32_cwd() calls eval which will recurse back
+            # here if we call abs_path, so we just assume it's normalised.
+            # warn "giving up on getting normalised filename from <$file>\n";
         }
         else
         {
             # print STDERR "getting abs_path <$file> ";
-            my $abs;
-            $abs = abs_path($file) unless -l $file;  # leave symbolic links
-            # print STDERR "giving <$file> ";
-            $file = $abs if defined $abs;
+            if (-e $file)  # Windows likes the file to exist
+            {
+                my $abs;
+                $abs = abs_path($file) unless -l $file;  # leave symbolic links
+                # print STDERR "giving <$abs> ";
+                $file = $abs if defined $abs;
+            }
         }
         # print STDERR "finally <$file> <$Dir>\n";
     }
     $file =~ s|\\|/|g if $^O eq "MSWin32";
-    $file =~ s|^$Dir/||;
+    $file =~ s|^$Dir/|| if defined $Dir;
 
     # print STDERR "File: $f => $file\n";
 
@@ -483,12 +492,16 @@ sub use_file
 {
     my ($file) = @_;
 
+    # warn "use_file($file)\n";
+
     # die "bad file" unless length $file;
 
-    $file = $1 if $file =~ /^\(eval \d+\)\[(.*):\d+\]/;
+    $file = $1 if $file =~ /^\(eval \d+\)\[(.+):\d+\]/;
     $file =~ s/ \(autosplit into .*\)$//;
 
     return $Files{$file} if exists $Files{$file};
+    return 0 if $file =~ /\(eval \d+\)/          ||
+                $file =~ /^\.\.[\/\\]\.\.[\/\\]lib[\/\\](?:Storable|POSIX).pm$/;
 
     my $f = normalised_file($file);
 
@@ -502,9 +515,7 @@ sub use_file
     # system "pwd; ls -l '$file'";
     $Files{$file} = -e $file ? 1 : 0;
     warn __PACKAGE__ . qq(: Can't find file "$file" (@_): ignored.\n)
-        unless $Files{$file} || $Silent || $file =~ /\(eval \d+\)/ ||
-               $file eq "../../lib/Storable.pm" ||
-               $file eq "../../lib/POSIX.pm";
+        unless $Files{$file} || $Silent;
 
     $Files{$file}
 }
@@ -542,7 +553,7 @@ sub B::GV::find_cv
 sub sub_info
 {
     my ($cv) = @_;
-    my ($name, $start) = ("", 0);
+    my ($name, $start) = ("--unknown--", 0);
     if (!$cv->GV->isa("B::SPECIAL"))
     {
         return unless $cv->GV->can("SAFENAME");
@@ -826,7 +837,7 @@ sub add_condition_cover
         # if exists $condition_locations{$$op};
 
     my $key = get_key($op);
-    # print STDERR "Condition cover $$op from $File:$Line\n";
+    # warn "Condition cover $$op from $File:$Line\n";
     # print STDERR "left:  [$left]\nright: [$right]\n";
     # use Carp "cluck"; cluck("from here");
 
@@ -1091,7 +1102,7 @@ sub get_cover
 
     ($Sub_name, my $start) = sub_info($cv);
 
-    # print STDERR "get_cover: <$Sub_name>\n";
+    # warn "get_cover: <$Sub_name>\n";
     return unless defined $Sub_name;  # Only happens within Safe.pm, AFAIK.
     # return unless length  $Sub_name;  # Only happens with Self_cover, AFAIK.
 
@@ -1284,7 +1295,7 @@ imagine that at least one of them is wrong.
 
 The most appropriate mailing list on which to discuss this module would
 be perl-qa.  Discussion has migrated there from perl-qa-metrics which is
-now defunct.  See L<http://lists.perl.org/showlist.cgi?name=perl-qa>.
+now defunct.  See L<http://lists.perl.org/list/perl-qa.html>.
 
 The Devel::Cover repository can be found at
 L<http://github.com/pjcj/Devel--Cover>.
@@ -1489,11 +1500,11 @@ See the BUGS file.  And the TODO file.
 
 =head1 VERSION
 
-Version 0.72 - 27th September 2010
+Version 0.74 - 16th April 2011
 
 =head1 LICENCE
 
-Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
