@@ -1,4 +1,4 @@
-# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,11 +10,11 @@ package Devel::Cover::DB;
 use strict;
 use warnings;
 
-our $VERSION = "0.73";
+our $VERSION = "0.74";
 
-use Devel::Cover::Criterion     0.73;
-use Devel::Cover::DB::File      0.73;
-use Devel::Cover::DB::Structure 0.73;
+use Devel::Cover::Criterion     0.74;
+use Devel::Cover::DB::File      0.74;
+use Devel::Cover::DB::Structure 0.74;
 
 use Carp;
 use File::Path;
@@ -189,8 +189,8 @@ sub is_valid
     for my $file (readdir $fh)
     {
         next if $file eq "." || $file eq "..";
-        next if ($file eq "runs" || $file eq "structure") &&
-                -e "$self->{db}/$file";
+        next if ($file eq "runs" || $file eq "structure" || $file eq "debuglog")
+                && -e "$self->{db}/$file";
         # warn "found $file in $self->{db}";
         return 0;
     }
@@ -1009,11 +1009,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.73 - 2nd October 2010
+Version 0.74 - 16th April 2011
 
 =head1 LICENCE
 
-Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2011, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
